#!/usr/bin/perl -w

$psfile = 'slides.ps';
$psoverlay = 'slides.mgp.ps';
$mgpfile = 'slides.mgp';
$bgcolor = '#dce4ff';

($width,$height,$sample) = (1024,768,4);
#($width,$height,$sample) = (1024,768,1);

pstomgp($psfile,$mgpfile);
psoverlay($psfile,$psoverlay);
pstogifs($psoverlay);

exit();

######################################################################

# Return an array that for each x has a list of the
# possible y where '%%Page: x y' is a line in the file

sub slide_pages { 
  my ($filename) = @_;
  my @slides = ([]);
  my ($slide,$page) = (0,0);

  open F,"$psfile";
  while (defined ($line = <F>)) {
    # Skip included EPS figure
    if ($line =~ /^%%BeginDocument:/) {
      while (defined ($line = <F>) and $line !~ /^%%(EOF|EndDocument)/) {}
    }
    elsif ($line =~ /%%Page:/) {
      ($slide,$page) = $line =~ /(\d+)/g;
      
      $slides[$slide] = [] if ! defined $slides[$slide];
      push @{$slides[$slide]},$page;
    }
  }
  close F;
  return @slides;
}

# Find number of pages in PostScript file

sub last_page {
  my ($filename) = @_;

  my @slides = slide_pages($filename);
  my $lastpage = map { @{$_} } @slides;
  return $lastpage;
}

# Incrementally put overlays on top of each other in PostScript file

sub psoverlay {
  my ($infile,$outfile) = @_;
  
  my @slides = slide_pages($infile);
  my $lastpage = last_page($infile);

  @pagelist = ();

  foreach $slide (1..$#slides) {
    foreach $i (0..$#{$slides[$slide]}) {
      push @pagelist, 
      join('+',
	   (map
	    { $_-1 } 
	    @{$slides[$slide]}[0..$i]
	   )
	  );
    }
  }
  $cmd = 'pstops -q "1:0@1.14(-1.8cm,-1.7cm)" < '.$infile.' | pstops "'.$lastpage.':'.join(',',@pagelist).'" > '.$outfile;
#  $cmd = "pstops \"$lastpage:".join(',',@pagelist)."\" $infile $outfile";
  print "$cmd\n";
  system $cmd;
}

# Convert PostScript file to a sequence of gif pictures

sub pstogifs {
  my ($infile) = @_;

  foreach $i (1..last_page($infile)) {
#  foreach $i (5) {
    $cmd =
      "psselect -p$i $infile"
     ." | gs -q -sDEVICE=ppmraw -g".($height * $sample)."x".($width * $sample)." -r".($width * $sample * 87 / 1024)." -sPAPERSIZE=a4 -sOutputFile=- -"
     ." | pnmscale ".(1/$sample)
     ." | ppmquant 256"
     ." | pnmflip -rotate90"
     ." | ppmtogif > $i.gif";
    print "$cmd\n";
    system $cmd;
  }
}

# Generate MagicPoint file

sub pstomgp {
  my ($psfile,$mgpfile) = @_;

  @slides = slide_pages($psfile);

  open MGP,">$mgpfile";
  print MGP "\%default 1 pcache 1 1 2 15, back \"$bgcolor\"\n";
  foreach $slide (1..$#slides) {
    @overlays = @{$slides[$slide]};
    print MGP "\%page\n";
    print MGP "\%image \"".$overlays[0].".gif\"\n";
    foreach $page (@overlays[1..$#overlays]) {
      print MGP "\%pcache 1 1 0 0\n";
      print MGP "\%page\n";
      print MGP "\%image \"$page.gif\"\n";
    }
  }
  close MGP;
}

#
